locals {
  protected_images_rules = <<EOF
%{ for index, tag in var.protected_images }
  {
      "rulePriority": ${index+1},
      "description": "Protect ${tag} - always keep latest version",
      "selection": {
        "tagStatus": "tagged",
        "countType": "imageCountMoreThan",
        "countNumber": 1,
        "tagPrefixList": ["${tag}"]
      },
      "action": {
        "type": "expire"
      }
    },
%{ endfor }
EOF
}

#
# Repository
#

resource "aws_ecr_repository" "default" {
  name = "${var.namespace}/${var.name}"
}

#
# Lifecycle:
#
# - remove untagged
# - keep max `max_images_count` dev images
# - keep max `max_images_count` RC images
# - keep max `max_images_count` release images
# - keep latest version of protected images
#
# https://docs.aws.amazon.com/AmazonECR/latest/userguide/LifecyclePolicies.html#lp_evaluation_rules
#

resource "aws_ecr_lifecycle_policy" "default" {
  repository = aws_ecr_repository.default.name

  policy = <<EOF
{
  "rules": [
${local.protected_images_rules}
    {
      "rulePriority": ${length(var.protected_images) + 1},
      "description": "Always keep ${var.max_images_count} release images stored.",
      "selection": {
        "tagStatus": "tagged",
        "tagPrefixList": ["${var.release_images_prefix}"],
        "countType": "imageCountMoreThan",
        "countNumber": ${var.max_images_count}
      },
      "action": {
        "type": "expire"
      }
    },
    {
      "rulePriority": ${length(var.protected_images) + 2},
      "description": "Always keep ${var.max_images_count} RC images stored.",
      "selection": {
        "tagStatus": "tagged",
        "tagPrefixList": ["${var.rc_images_prefix}"],
        "countType": "imageCountMoreThan",
        "countNumber": ${var.max_images_count}
      },
      "action": {
        "type": "expire"
      }
    },
    {
      "rulePriority": ${length(var.protected_images) + 3},
      "description": "Remove untagged images.",
      "selection": {
        "tagStatus": "untagged",
        "countType": "imageCountMoreThan",
        "countNumber": 1
      },
      "action": {
        "type": "expire"
      }
    },
    {
      "rulePriority": ${length(var.protected_images) + 4},
      "description": "Remove dev images when reached ${var.max_images_count} images stored.",
      "selection": {
        "tagStatus": "any",
        "countType": "imageCountMoreThan",
        "countNumber": ${var.max_images_count}
      },
      "action": {
        "type": "expire"
      }
    }
  ]
}
EOF
}
