resource "aws_lb" "default" {
  load_balancer_type = "network"
  name               = "${var.name}-${var.env}-nlb"
  subnets            = var.jz_vpc.subnets.*.id

  enable_deletion_protection = true

  tags = {
    Name = "${var.name}-${var.env}-nlb"
    Env  = var.env
  }

  lifecycle {
    ignore_changes = [
      tags,
    ]
  }
}

locals {
  # Mapping of NLB listeners from config port_mapping.
  listeners = {
    for i, z in var.port_mapping:
    z.nlb_port => {
      app_port     = z.app_port
      app_protocol = upper(z.app_protocol)
      nlb_port     = z.nlb_port
      nlb_protocol = upper(z.nlb_protocol)
      is_secure    = upper(z.nlb_protocol) == "TLS"
    }...
  }
  # Mapping of NLB target groups from config port_mapping.
  target_groups = {
    for i, z in var.port_mapping:
    z.app_port => {
      app_port     = z.app_port
      app_protocol = upper(z.app_protocol)
    }...
  }
  # TGs * instances
  instance_ids_flatten = flatten([
    var.jz_swarm_project.instance_ids.leader,
    var.jz_swarm_project.instance_ids.managers.*,
    var.jz_swarm_project.instance_ids.workers.*]
  )
  target_group_arns = values(aws_lb_target_group.default).*.arn
  tg_x_instance     = setproduct(local.target_group_arns, local.instance_ids_flatten)
}

resource "aws_lb_listener" "default" {
  for_each = local.listeners

  load_balancer_arn = aws_lb.default.arn
  port              = each.value[0].nlb_port
  protocol          = each.value[0].nlb_protocol
  ssl_policy        = each.value[0].is_secure ? "ELBSecurityPolicy-2016-08" : null
  certificate_arn   = each.value[0].is_secure ? data.aws_acm_certificate.default[0].arn : null

  default_action {
    type             = "forward"
    target_group_arn = aws_lb_target_group.default[each.value[0].app_port].arn
  }
}

resource "aws_lb_target_group" "default" {
  for_each = local.target_groups

  name     = "${var.name}-${var.env}-${each.key}-${each.value[0].app_protocol}-tg"
  vpc_id   = var.jz_vpc.self.id

  port     = each.value[0].app_port
  protocol = each.value[0].app_protocol

  health_check {
      enabled  = true
      port     = var.health_check_port
      protocol = each.value[0].app_protocol
  }
}

resource "aws_lb_target_group_attachment" "default" {
  count = length(local.tg_x_instance)

  target_group_arn = local.tg_x_instance[count.index][0]
  target_id        = local.tg_x_instance[count.index][1]
}

data "aws_acm_certificate" "default" {
  count       = var.ssl_cert_domain != "" ? 1 : 0
  domain      = var.ssl_cert_domain
  most_recent = true
}
